/*  Copyright (c) 2007-2008 Tomasz Moń <desowin@gmail.com>
 *  Copyright (c) 2004 Artur Gajda
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; under version 3 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses>.
 */

#include <libintl.h>
#include "riv.h"
#include "ui_priv.h"
#include "net.h"
#include "lista.h"
#include "ui.h"
#include "conf.h"

GList *priv_lista = NULL;

void priv_new(char *nick, gint id) {
    PRIV *priv = NULL;
    priv = priv_lista_find_by_nick(nick);

    if ( priv == NULL )
        priv = priv_new_window(nick, id);
    else
        gtk_window_present(GTK_WINDOW(priv->window));
}

int priv_new_message(char *nick, gint id, char *msg, char *tag) {
    PRIV *priv = NULL;
    //search for open window, first by id, if that fails by nick...
    priv = priv_lista_find_by_id(id);
    if (priv == NULL)
        priv = priv_lista_find_by_nick(nick);

    if (priv == NULL)
        priv = priv_new_window(nick, id); //create new window

    priv_add_text2(nick, msg, priv, tag);
    if (!GTK_WIDGET_VISIBLE(priv->window))
        gtk_widget_show(priv->window);
    return 0;
}


PRIV *priv_lista_find_by_nick(const char *nick) {
    PRIV *priv;

    GList *iter;
    for (iter = priv_lista; iter; iter = g_list_next(iter)) {
        priv = (PRIV*) iter->data;
        if ( strcmp(priv->nick, nick) == 0 ) {
            return priv;
        }
    }

    //printf("nie znaleziono %s\n", nick);
    return NULL;
}

PRIV *priv_lista_find_by_id(gint id) {
    PRIV *priv;

    GList *iter;
    for (iter = priv_lista; iter; iter = g_list_next(iter)) {
        priv = (PRIV*) iter->data;
        if (priv->id == id) {
            return priv;
        }
    }

    return NULL;
}

void priv_add_text(char *txt, PRIV *priv) {
    GtkTextIter iter, iter2;
    GtkTextBuffer *buffer;
    GtkTextMark* mark;

    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (priv->view));
    gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
    mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, TRUE);
    gtk_text_buffer_insert(buffer, &iter, txt, strlen(txt));
    gtk_text_buffer_get_end_iter(buffer, &iter);
    gtk_text_buffer_get_iter_at_mark(buffer, &iter2, mark);
    gtk_text_buffer_delete_mark(buffer, mark);
    gtk_text_buffer_apply_tag_by_name(buffer, "ITALICS", &iter, &iter2);

    gtk_text_buffer_get_iter_at_offset (buffer, &iter, -1);
    mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, 1);
    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(priv->view), mark, 0.0, 0, 0.0, 1.0);
    gtk_text_buffer_delete_mark (buffer, mark);
}

void priv_set_zombie(PRIV *priv, gint typ) {
    gchar *title = g_strdup_printf("%s - %s [zombie]", "griv", priv->nick);
    gtk_window_set_title(GTK_WINDOW(priv->window), title);
    g_free(title);
    gchar *txt = NULL;
    switch(typ) {
        case ZOMBIE_TIMEOUT:
        txt = g_strdup_printf("%s disconnected.\n", priv->nick);
        break;
        case ZOMBIE_QUIT:
        txt = g_strdup_printf("%s has left.\n", priv->nick);
        break;
    }
    if (txt) {
        priv_add_text(txt, priv);
        g_free(txt);
    }
}

void priv_add_text2(char *nick, char *txt, PRIV *priv, char *tag) {
    GtkTextIter iter;
    GtkTextBuffer *buffer;
    GtkTextMark* mark;

    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (priv->view));
    gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
    gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, nick, -1, "BOLD", tag, NULL);

    gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
    gchar *tmp = g_strdup_printf(": %s\n", txt);
    gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, tmp, -1, tag, NULL);
    g_free(tmp);

    gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
    mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, 1);
    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(priv->view), mark, 0.0, 0, 0.0, 1.0);
    gtk_text_buffer_delete_mark (buffer, mark);
}

static void priv_on_enter( GtkWidget *widget, PRIV *priv) {
    gchar *txt;
    txt = g_strdup(gtk_entry_get_text(GTK_ENTRY(priv->edit)));
    if (strlen(txt) < 1) {
        g_free(txt);
        return;
    }

    gchar *tag = NULL;
    if (conf.display_colors)
        tag = g_strdup_printf("color-%s", conf.nick);
    if ( send_priv_message(txt, priv->id) == -1 ) {
        priv_add_text2("!!!", _("Message wasn't sent"), priv, NULL);
    } else
        priv_add_text2("Ja", txt, priv, tag);

    g_free(txt);
    gtk_entry_set_text(GTK_ENTRY(priv->edit), "");
}

gint priv_delete_event( GtkWidget *widget, GdkEvent  *event, PRIV *priv ) {
    gtk_widget_hide(widget);
    return TRUE;
}

PRIV *priv_new_window(const char *nick, gint id) {
    GtkWidget *vbox, *sw;
    GtkTextBuffer *buffer;

    PRIV *priv = g_new0(PRIV, 1);

    priv->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    priv->nick = g_strdup(nick);
    priv->id = id;

    gchar *tmp = g_strdup_printf("%s - %s", "griv", nick);
    gtk_window_set_title (GTK_WINDOW (priv->window), tmp);
    g_free(tmp);

    g_signal_connect (G_OBJECT (priv->window), "delete_event",
    G_CALLBACK (priv_delete_event), priv);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (priv->window), vbox);

    buffer = gtk_text_buffer_new(griv_tag_table);
    priv->view = gtk_text_view_new_with_buffer(buffer);
    gtk_text_buffer_set_text(buffer, _("Not encrypted transmission!\n"), -1);
    gtk_text_view_set_editable(GTK_TEXT_VIEW (priv->view), FALSE);
    gtk_widget_set_size_request(priv->view, 335, 220);
    //gtk_text_buffer_create_tag(buffer, "BOLD", "weight", PANGO_WEIGHT_BOLD, NULL);
    //gtk_text_buffer_create_tag(buffer, "ITALICS", "style", PANGO_STYLE_ITALIC, NULL);
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(priv->view), GTK_WRAP_WORD_CHAR);

    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(sw), priv->view);
    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);

    priv->edit = gtk_entry_new();
    gtk_box_pack_end(GTK_BOX(vbox), priv->edit, FALSE, TRUE, 0);
    g_signal_connect(G_OBJECT(priv->edit), "activate", G_CALLBACK(priv_on_enter), priv);

    priv_lista = g_list_append(priv_lista, priv);
    gtk_widget_grab_focus(priv->edit);
    gtk_widget_show_all(priv->window);

    return priv;
}
